module ApplicationHelper
  # Return a title on a per-page basis.
  def title
    base_title = 'Unipept'
    if @title.nil?
      base_title
    else
      "#{@title} | #{base_title}"
    end
  end

  # helper to add remove links to forms
  def link_to_remove_fields(name, f)
    f.hidden_field(:_destroy) + link_to(name, '#', onclick: 'remove_fields(this); return false;')
  end

  # helper to add add links to forms
  def link_to_add_fields(name, f, association)
    new_object = f.object.class.reflect_on_association(association).klass.new
    fields = f.fields_for(association, new_object, child_index: "new_#{association}") do |builder|
      render(association.to_s.singularize + '_fields', f: builder)
    end
    link_to(name, '#', onclick: "add_fields(this, \"#{association}\", \"#{escape_javascript(fields)}\"); return false;")
  end

  class BootstrapLinkRenderer < ::WillPaginate::ActionView::LinkRenderer
    protected

      def html_container(html)
        tag :ul, html, container_attributes
      end

      def page_number(page)
        tag :li, link(page, page, rel: rel_value(page)), class: ('active' if page == current_page)
      end

      def gap
        tag :li, link('&hellip;', '#'), class: 'disabled'
      end

      def previous_or_next_page(page, text, classname)
        tag :li, link(text, page || '#'), class: [classname[0..3], classname, ('disabled' unless page)].join(' ')
      end
  end

  def page_navigation_links(pages)
    will_paginate(pages, class: 'pagination', inner_window: 2, outer_window: 0, renderer: BootstrapLinkRenderer, previous_label: '&larr;'.html_safe, next_label: '&rarr;'.html_safe)
  end
end
